---------------------------------------------------
-- Structured baud rate generator for the UART peripheral
-- Count Enable (CE) and asynchronous active-high clear direct (CD)
-- 4 cascaded modules working in series to produce the following 
-- outputs from the Altera DE0 board's 50 MHz oscillator.
-- OSC_CLK_out  (divider by 1) --> 50 MHz
-- RX_CLK (divider by "n" from the input OSC_CLK_in). n = 651 to generate a (9600 b/s) x 8 clock for the receiver
-- TX_CLK (divider by 8)
-- CLK_100Hz (divider by 96 for example, to generate the 100 Hz cloxk for the debouncing filter)
-- CLK_1Hz_SQUARED (divider by 100 using internal components (a divider by 50 and a T-FF)
-- Schematic block diagram at: http://digsys.upc.es/ed//CSD/terms/1112Q1/EX/EX2/baud_rate_generator_schematic.DSN
---------------------------------------------------                                                              
-- NOTE: The CD has been made active-high like the other modules in the UART
-- CD as active low in the EX2 (CD_L)
---------------------------------------------------

LIBRARY IEEE;
   USE IEEE.STD_LOGIC_1164.ALL;
   USE IEEE.STD_LOGIC_UNSIGNED.ALL;
	
ENTITY BAUD_RATE_GENERATOR IS  
      PORT(	CD,CE				: 	IN  std_logic;
			OSC_CLK_in			: 	IN  std_logic;
			OSC_CLK_out			:	OUT std_logic;
			RX_CLK				:	OUT std_logic;         
			TX_CLK				:	OUT std_logic;
			CLK_100Hz			:	OUT std_logic;
			CLK_1Hz_SQUARED		:	OUT std_logic
--			DISPLAYS			:	OUT std_logic_vector(14 downto 0;
-- This DISPLAYS vector is an extra output only to blank unused LED's and segments		
-- In this example, only the decimal point of the Digit 2 is used, so better keep all the other LED's off.  
          );
END BAUD_RATE_GENERATOR ;

ARCHITECTURE schematic OF BAUD_RATE_GENERATOR  IS  
-- Components

	COMPONENT RECEIVER_CLOCK_DIVIDER IS  
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC651		:	OUT std_logic
		);
	END COMPONENT;


	COMPONENT TRANSMITTER_CLOCK_DIVIDER IS  
		PORT(	CD,CLK,CE	: 	IN  std_logic;
					TC8		:	OUT std_logic
		);
	END COMPONENT;				
	
	COMPONENT SWITCH_DEBOUNCING_DIVIDER IS                 
		PORT(	CD,CLK,CE		: 	IN  std_logic;
				TC96				:	OUT std_logic
		);
	END COMPONENT;	
  
	COMPONENT SQUARE_1HZ_DIVIDER IS  
		PORT(	CD,CLK,CE		: 	IN  std_logic;
				TC100				:	OUT std_logic
		);
	END COMPONENT;
	
	
-- Signals for connecting components together (just the internal wires)
	SIGNAL OSC_CLK				:	std_logic;
	SIGNAL RX_CLK1, TX_CLK1, CLK3	:	std_logic;
		
	BEGIN  
-- Instantiation of components

	CHIP1	: 	RECEIVER_CLOCK_DIVIDER
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> CE,
			TC651	=> RX_CLK1					  
		);

	CHIP2	: 	TRANSMITTER_CLOCK_DIVIDER
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> RX_CLK1,
			TC8 	=> TX_CLK1					  
		);

	CHIP3	: 	SWITCH_DEBOUNCING_DIVIDER
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> TX_CLK1,
			TC96 	=> CLK3					  
		);

	CHIP4	:	SQUARE_1HZ_DIVIDER
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> CLK3,
			TC100 	=> CLK_1Hz_SQUARED					  
		);
		
-- connections and logic between components
	
	-- The circuit's signals that have to be connected to input ports 
	OSC_CLK <= OSC_CLK_in;
	
	-- The output ports that have to be connected to internal signals 
	OSC_CLK_out	<= OSC_CLK;
	RX_CLK		<= RX_CLK1;
	TX_CLK		<= TX_CLK1;
	CLK_100Hz	<= CLK3;
	
END schematic ;
